@@ -28,7 +28,7 @@ else
     }
 fi
 
-if type gettext > /dev/null 2>&1; then
+if which gettext > /dev/null 2>&1; then
     _() {
         gettext "$@"
     }
@@ -106,6 +106,17 @@ detectDE() {
 }
 
 run_kde() {
+    if xprop -root KDE_SESSION_VERSION 2>&1 | grep -q "= 5"; then
+        if (kcmshell5 --list 2>/dev/null | grep ^kcm_fcitx > /dev/null 2>&1); then
+            if [ x"$1" != x ]; then
+                exec kcmshell5 kcm_fcitx --args "$1"
+            else
+                exec kcmshell5 kcm_fcitx
+            fi
+        fi
+    fi
+
+    # not harm to also check kde4 version on kf5
     if (kcmshell4 --list 2>/dev/null | grep ^kcm_fcitx > /dev/null 2>&1); then
         if [ x"$1" != x ]; then
             exec kcmshell4 kcm_fcitx --args "$1"
@@ -193,7 +204,7 @@ detectDE
 # even if we are not on KDE, we should still try kde, some wrongly
 # configured kde desktop cannot be detected (usually missing xprop)
 # and if kde one can work, so why not use it if gtk, gtk3 not work?
-# xdg/editor is never a prefered solution
+# xdg/editor is never a preferred solution
 case "$DE" in
     kde)
         order="kde gtk3 gtk xdg editor"
