@@ -11,6 +11,7 @@ typedef enum _XimCallType {
 
 typedef struct _XimQueue XimQueue;
 
-void XimQueueInit(FcitxXimFrontend* xim);
-void* XimConsumeQueue(void* arg, FcitxModuleFunctionArg args);
-void XimPendingCall(FcitxXimFrontend* xim, XimCallType type, XPointer ptr);
\ No newline at end of file
+void XimQueueInit(FcitxXimFrontend *xim);
+void XimConsumeQueue(FcitxXimFrontend *xim);
+void XimPendingCall(FcitxXimFrontend *xim, XimCallType type, XPointer ptr);
+void XimQueueDestroy(FcitxXimFrontend *xim);
