@@ -91,7 +91,7 @@ void LoadSPData(FcitxPinyinState *pystate)
         i = 0;
 
         while (SPMap_C[i].strQP[0]) {
-            if (strlen(SPMap_C[i].strQP) == 1)
+            if (!SPMap_C[i].strQP[1])
                 SPMap_C[i].cJP = SPMap_C[i].strQP[0];
             i ++ ;
         }
@@ -112,7 +112,7 @@ void LoadSPData(FcitxPinyinState *pystate)
             if (*pstr == ' ' || *pstr == '\t')
                 pstr++;
 
-            if (!strlen(pstr) || pstr[0] == '#')
+            if (*pstr == '\0' || *pstr == '#')
                 continue;
 
             if (!strncmp(pstr, cstr(NAME), cstrlen(NAME))) {
@@ -203,7 +203,7 @@ void LoadSPData(FcitxPinyinState *pystate)
 /*
  * 此处只转换单个双拼，并且不检查错误
  */
-void SP2QP(FcitxPinyinConfig* pyconfig, char *strSP, char *strQP)
+void SP2QP(FcitxPinyinConfig* pyconfig, const char *strSP, char *strQP)
 {
     int             iIndex1 = 0, iIndex2 = 0;
     char            strTmp[2];
