@@ -39,7 +39,9 @@ struct _FcitxMemoryPool {
 };
 
 static void fcitx_memory_chunk_free(void* c);
-const UT_icd chunk_icd = { sizeof(FcitxMemoryChunk), NULL, NULL, fcitx_memory_chunk_free};
+static const UT_icd chunk_icd = {
+    sizeof(FcitxMemoryChunk), NULL, NULL, fcitx_memory_chunk_free
+};
 
 void fcitx_memory_chunk_free(void* c) {
     FcitxMemoryChunk* chunk = (FcitxMemoryChunk*) c;
@@ -61,11 +63,7 @@ FcitxMemoryPool* fcitx_memory_pool_create()
 static inline void*
 memory_align_ptr(void *p)
 {
-    intptr_t res = (intptr_t)p;
-    int left;
-    if ((left = res % sizeof(int)))
-        return p + sizeof(int) - left;
-    return p;
+    return (void*)fcitx_utils_align_to((uintptr_t)p, sizeof(int));
 }
 
 FCITX_EXPORT_API
@@ -99,13 +97,15 @@ void* fcitx_memory_pool_alloc_align(FcitxMemoryPool* pool, size_t size, int alig
 
     if (chunk->end - chunk->cur <= FCITX_MEMORY_CHUNK_FULL_SIZE) {
         utarray_push_back(pool->fullchunks, chunk);
-        int idx = utarray_eltidx(pool->chunks, chunk);
+        unsigned int idx = utarray_eltidx(pool->chunks, chunk);
         utarray_remove_quick(pool->chunks, idx);
     }
 
     return result;
 }
 
+#undef fcitx_memory_pool_alloc
+
 FCITX_EXPORT_API
 void* fcitx_memory_pool_alloc(FcitxMemoryPool* pool, size_t size)
 {
