@@ -254,6 +254,10 @@ typedef struct {
         (a)->i -= (len);                                                \
     } while(0)
 
+/**
+ * this actually steal the content away, doesn't destroy the content
+ * so the name doesn't actually accurate, use it carefully.
+ */
 #define utarray_remove_quick(a, pos) do {                               \
         if ((a)->i - 1 != (pos))                                        \
             memcpy(_utarray_eltptr(a, pos), _utarray_eltptr(a, (a)->i - 1), \
@@ -261,6 +265,19 @@ typedef struct {
         (a)->i--;                                                       \
     } while(0)
 
+/**
+ * this is the "real" remove_quick with destroy,
+ */
+#define utarray_remove_quick_full(a, pos) do {                          \
+        if ((a)->icd->dtor) {                                           \
+            (a)->icd->dtor(utarray_eltptr(a, pos));                     \
+        }                                                               \
+        if ((a)->i - 1 != (pos))                                        \
+            memcpy(_utarray_eltptr(a, pos), _utarray_eltptr(a, (a)->i - 1), \
+                   (a)->icd->sz);                                       \
+        (a)->i--;                                                       \
+    } while(0)
+
 #define utarray_clear(a) do {                                   \
         if ((a)->i > 0) {                                       \
             if ((a)->icd->dtor) {                               \
