@@ -174,6 +174,14 @@ extern "C" {
      **/
     boolean FcitxInstanceSetCurrentIC(struct _FcitxInstance* instance, FcitxInputContext* ic);
 
+    /**
+     * Get last focused input context.
+     *
+     * @param instance
+     * @return last focused input context
+     */
+    FcitxInputContext* FcitxInstanceGetLastIC(FcitxInstance* instance);
+
     /**
      * Get Addons From Instance
      *
@@ -277,6 +285,8 @@ extern "C" {
      **/
     boolean FcitxInstanceRun(int argc, char* argv[], int fd);
 
+    void FcitxInstanceRestart(FcitxInstance *instance);
+
 #ifdef __cplusplus
 }
 #endif
