@@ -148,7 +148,8 @@ void *ChttransCreate(FcitxInstance* instance)
     FcitxInstanceRegisterCommitFilter(instance, shk);
     FcitxInstanceRegisterIMChangedHook(instance, imhk);
     FcitxUIRegisterStatus(instance, transState, "chttrans",
-                          ChttransEnabled(transState) ? _("Traditional Chinese") :  _("Simplified Chinese"),
+                          ChttransEnabled(transState) ? _("Switch to Simplified Chinese") :
+                          _("Switch to Traditional Chinese"),
                           _("Toggle Simp/Trad Chinese Conversion"),
                           ToggleChttransState,
                           GetChttransEnabled);
@@ -186,15 +187,15 @@ void ToggleChttransState(void* arg)
 {
     FcitxChttrans *transState = (FcitxChttrans*)arg;
     FcitxInstance *instance = transState->owner;
-    FcitxIM* im = FcitxInstanceGetCurrentIM(transState->owner);
+    FcitxIM* im = FcitxInstanceGetIM(transState->owner, FcitxInstanceGetLastIC(transState->owner));
     if (!im)
         return;
     boolean enabled = !ChttransEnabled(transState);
 
     fcitx_string_map_set(transState->enableIM, im->uniqueName, enabled);
     FcitxUISetStatusString(instance, "chttrans",
-                           enabled ? _("Traditional Chinese") :
-                           _("Simplified Chinese"),
+                           enabled ? _("Switch to Simplified Chinese") :
+                           _("Switch to Traditional Chinese"),
                           _("Toggle Simp/Trad Chinese Conversion"));
     FcitxUIUpdateInputWindow(instance);
     SaveChttransConfig(transState);
@@ -238,7 +239,11 @@ void ChttransIMChanged(void* arg)
     FcitxIM* im = FcitxInstanceGetCurrentIM(transState->owner);
     if (!im)
         return;
-    FcitxUIRefreshStatus(transState->owner, "chttrans");
+    boolean enabled = ChttransEnabled(transState);
+    FcitxUISetStatusString(transState->owner, "chttrans",
+                           enabled ? _("Switch to Simplified Chinese") :
+                           _("Switch to Traditional Chinese"),
+                          _("Toggle Simp/Trad Chinese Conversion"));
 }
 
 /**
