@@ -348,6 +348,7 @@ ClipboardPostHook(void *arg, FcitxKeySym sym, unsigned int state,
     FcitxCandidateWordSetPageSize(cand_list, page_size);
     FcitxCandidateWordSetChooseAndModifier(
         cand_list, DIGIT_STR_CHOOSE, cmodifiers[config->choose_modifier]);
+    FcitxCandidateWordSetOverrideDefaultHighlight(cand_list, false);
     if (clipboard->clp_hist_len) {
         ClipboardSetCandWord(clipboard, &cand_word, clipboard->clp_hist_lst);
         FcitxCandidateWordAppend(cand_list, &cand_word);
@@ -378,6 +379,8 @@ skip_primary:
         FcitxCandidateWordAppend(cand_list, &cand_word);
     }
     *ret_val = IRV_FLAG_UPDATE_INPUT_WINDOW;
+    FcitxCandidateWordSetType(FcitxCandidateWordGetFirst(cand_list),
+                              MSG_CANDIATE_CURSOR);
     return true;
 }
 
@@ -404,15 +407,19 @@ ClipboardCreate(FcitxInstance *instance)
 /* #endif */
     ApplyClipboardConfig(clipboard);
 
-    FcitxKeyFilterHook key_hook = {
-        .arg = clipboard,
-        .func = ClipboardPreHook
-    };
+    FcitxKeyFilterHook key_hook;
+
+    key_hook.arg = clipboard;
+    key_hook.func = ClipboardPreHook;
     FcitxInstanceRegisterPreInputFilter(instance, key_hook);
 
     key_hook.func = ClipboardPostHook;
     FcitxInstanceRegisterPostInputFilter(instance, key_hook);
 
+    key_hook.arg = &clipboard->active;
+    key_hook.func = FcitxDummyReleaseInputHook;
+    FcitxInstanceRegisterPreReleaseInputFilter(instance, key_hook);
+
     FcitxIMEventHook reset_hook = {
         .arg = clipboard,
         .func = ClipboardReset
@@ -427,6 +434,7 @@ ClipboardDestroy(void *arg)
 {
     FcitxClipboard *clipboard = (FcitxClipboard*)arg;
     ClipboardWriteHistory(clipboard);
+    FcitxConfigFree(&clipboard->config.gconfig);
     fcitx_utils_free(clipboard->primary.str);
     free(arg);
 }
