@@ -110,8 +110,12 @@ const char *kFcitxLua =
     "ime.get_last_commit = function()"
     "    return __ime_get_last_commit();"
     "end;";
-const UT_icd FunctionItem_icd = {sizeof(FunctionItem), NULL, FunctionItemCopy, FunctionItemDtor};
-const UT_icd LuaResultItem_icd = {sizeof(LuaResultItem), NULL, LuaResultItemCopy, LuaResultItemDtor};
+static const UT_icd FunctionItem_icd = {
+    sizeof(FunctionItem), NULL, FunctionItemCopy, FunctionItemDtor
+};
+static const UT_icd LuaResultItem_icd = {
+    sizeof(LuaResultItem), NULL, LuaResultItemCopy, LuaResultItemDtor
+};
 
 LuaModule * LuaModuleAlloc(FcitxInstance *fcitx) {
     LuaModule *module;
@@ -200,7 +204,7 @@ static int ImeRegisterTrigger_Export(lua_State *lua) {
             return 0;
         }
         size_t i;
-        size_t len = lua_objlen(lua, kInputStringsArg);
+        size_t len = luaL_len(lua, kInputStringsArg);
         for (i = 1; i <= len; ++i) {
             lua_pushinteger(lua, i);
             lua_gettable(lua, kInputStringsArg);
@@ -248,10 +252,11 @@ static void LuaResultItemDtor(void *_elt) {
 static void FreeTrigger(TriggerItem **triggers, LuaExtension *extension) {
     TriggerItem *trigger;
     for (trigger = *triggers; trigger != NULL; ) {
-        int count = utarray_len(trigger->functions);
-        int i;
-        for (i = 0; i < count; ) {
-            FunctionItem *f = (FunctionItem *)utarray_eltptr(trigger->functions, i);
+        unsigned int count = utarray_len(trigger->functions);
+        unsigned int i;
+        FunctionItem *f;
+        for (i = 0;i < count;) {
+            f = (FunctionItem*)utarray_eltptr(trigger->functions, i);
             if (f->lua == extension->lua) {
                 utarray_erase(trigger->functions, i, 1);
                 --count;
@@ -614,7 +619,7 @@ static UT_array * LuaCallFunction(lua_State *lua,
                                   const char *function_name,
                                   const char *argument) {
     UT_array *result = NULL;
-    lua_getfield(lua, LUA_GLOBALSINDEX, "__ime_call_function");
+    lua_getglobal(lua, "__ime_call_function");
     lua_pushstring(lua, function_name);
     lua_pushstring(lua, argument);
     int rv = lua_pcall(lua, 2, 1, 0);
@@ -639,7 +644,7 @@ static UT_array * LuaCallFunction(lua_State *lua,
             FcitxLog(WARNING, "lua function return return null");
         }
     } else if (type == LUA_TTABLE) {
-        size_t i, len = lua_objlen(lua, -1);
+        size_t i, len = luaL_len(lua, -1);
         if (len < 1) {
             return result;
         }
@@ -712,7 +717,9 @@ static UT_array * LuaCallFunction(lua_State *lua,
     return result;
 }
 
-UT_array * InputCommand(LuaModule *module, const char *input) {
+UT_array *
+InputCommand(LuaModule *module, const char *input)
+{
     CommandItem *command;
     char key[3];
     strncpy(key, input, sizeof(key));
@@ -732,7 +739,7 @@ UT_array * InputCommand(LuaModule *module, const char *input) {
 
 UT_array * InputTrigger(LuaModule *module, const char *input) {
     if (module->shortest_input_trigger_key_length == 0
-            || strlen(input) < module->shortest_input_trigger_key_length) {
+        || strlen(input) < module->shortest_input_trigger_key_length) {
         return NULL;
     }
     TriggerItem *trigger;
