@@ -179,6 +179,7 @@ void* FcitxNotificationItemCreate(FcitxInstance* instance)
 {
     FcitxNotificationItem* notificationitem = fcitx_utils_new(FcitxNotificationItem);
     notificationitem->owner = instance;
+    notificationitem->revision = 2;
     DBusError err;
     dbus_error_init(&err);
     do {
@@ -190,12 +191,6 @@ void* FcitxNotificationItemCreate(FcitxInstance* instance)
 
         notificationitem->conn = conn;
 
-        DBusObjectPathVTable fcitxIPCVTable = {NULL, &FcitxNotificationItemEventHandler, NULL, NULL, NULL, NULL };
-        if (!dbus_connection_register_object_path(notificationitem->conn, NOTIFICATION_ITEM_DEFAULT_OBJ, &fcitxIPCVTable, notificationitem)) {
-            FcitxLog(ERROR, "No memory");
-            break;
-        }
-
         if (!FcitxDBusMenuCreate(notificationitem)) {
             FcitxLog(ERROR, "No memory");
             break;
@@ -260,9 +255,12 @@ void FcitxNotificationItemDestroy(void* arg)
 {
     FcitxNotificationItem* notificationitem = (FcitxNotificationItem*) arg;
     if (notificationitem->conn) {
-        dbus_connection_unregister_object_path(notificationitem->conn, NOTIFICATION_ITEM_DEFAULT_OBJ);
+        if (notificationitem->callback) {
+            dbus_connection_unregister_object_path(notificationitem->conn, NOTIFICATION_ITEM_DEFAULT_OBJ);
+        }
         dbus_connection_unregister_object_path(notificationitem->conn, "/MenuBar");
     }
+    notificationitem->ids = MenuIdSetClear(notificationitem->ids);
 
     free(notificationitem);
 }
@@ -363,16 +361,17 @@ DBusHandlerResult FcitxNotificationItemEventHandler (DBusConnection  *connection
 char* FcitxNotificationItemGetIconNameString(FcitxNotificationItem* notificationitem)
 {
     char* iconName = NULL;
-    FcitxIM* im = FcitxInstanceGetCurrentIM(notificationitem->owner);
-    const char* icon = "";
+    FcitxIM* im = FcitxInstanceGetIM(notificationitem->owner, FcitxInstanceGetLastIC(notificationitem->owner));
+    const char* icon = NULL;
     if (im) {
         if (strncmp(im->uniqueName, "fcitx-keyboard-",
                     strlen("fcitx-keyboard-")) != 0) {
             icon = im->strIconName;
-        } else {
-            return strdup("input-keyboard");
         }
     }
+    if (!icon) {
+        return strdup("input-keyboard");
+    }
     boolean result = CheckAddPrefix(&icon);
     fcitx_utils_alloc_cat_str(iconName, result ? "fcitx-" : "", icon);
     return iconName;
@@ -415,14 +414,13 @@ void FcitxNotificationItemGetTitle(void* arg, DBusMessageIter* iter)
 void FcitxNotificationItemGetIconName(void* arg, DBusMessageIter* iter)
 {
     FcitxNotificationItem* notificationitem = (FcitxNotificationItem*) arg;
-    FcitxInputContext* ic = FcitxInstanceGetCurrentIC(notificationitem->owner);
-    if (ic == NULL) {
+    char* icon = FcitxNotificationItemGetIconNameString(notificationitem);
+    if (!icon) {
         const char* iconName = "input-keyboard";
         dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &iconName);
     } else {
-        char* iconName = FcitxNotificationItemGetIconNameString(notificationitem);
-        dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &iconName);
-        free(iconName);
+        dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &icon);
+        free(icon);
     }
 }
 
@@ -458,7 +456,7 @@ void FcitxNotificationItemGetToolTip(void* arg, DBusMessageIter* iter)
     } else {
         iconName = FcitxNotificationItemGetIconNameString(notificationitem);
         iconNameToFree = iconName;
-        FcitxIM* im = FcitxInstanceGetCurrentIM(notificationitem->owner);
+        FcitxIM* im = FcitxInstanceGetIM(notificationitem->owner, FcitxInstanceGetLastIC(notificationitem->owner));
         title = im ? im->strName : _("Disabled");
         content = im ? "" : _("Input Method Disabled");
     }
@@ -478,7 +476,7 @@ const char* FcitxNotificationItemGetLabel(FcitxNotificationItem* notificationite
 
     FcitxInputContext* ic = FcitxInstanceGetCurrentIC(notificationitem->owner);
     if (ic) {
-        FcitxIM* im = FcitxInstanceGetCurrentIM(notificationitem->owner);
+        FcitxIM* im = FcitxInstanceGetIM(notificationitem->owner, FcitxInstanceGetLastIC(notificationitem->owner));
         if (im) {
             if (strncmp(im->uniqueName, "fcitx-keyboard-",
                         strlen("fcitx-keyboard-")) == 0) {
@@ -585,6 +583,9 @@ boolean FcitxNotificationItemEnable(FcitxNotificationItem* notificationitem, Fci
         FcitxLog(ERROR, "This should not happen, please report bug.");
         return false;
     }
+
+    DBusObjectPathVTable fcitxIPCVTable = {NULL, &FcitxNotificationItemEventHandler, NULL, NULL, NULL, NULL };
+    dbus_connection_register_object_path(notificationitem->conn, NOTIFICATION_ITEM_DEFAULT_OBJ, &fcitxIPCVTable, notificationitem);
     notificationitem->callback = callback;
     notificationitem->data = data;
     asprintf(&notificationitem->serviceName, "org.kde.StatusNotifierItem-%u-%d", getpid(), ++notificationitem->index);
@@ -610,6 +611,9 @@ boolean FcitxNotificationItemEnable(FcitxNotificationItem* notificationitem, Fci
 
 void FcitxNotificationItemDisable(FcitxNotificationItem* notificationitem)
 {
+    if (notificationitem->callback) {
+        dbus_connection_unregister_object_path(notificationitem->conn, NOTIFICATION_ITEM_DEFAULT_OBJ);
+    }
     notificationitem->callback = NULL;
     notificationitem->data = NULL;
 
