@@ -64,7 +64,7 @@ CharFromPhraseModeGetCandCb(void *arg, FcitxCandidateWord *cand_word)
     int len2 = strlen(cand_word->strWord);
     pyenhance->cfp_mode_selected = realloc(pyenhance->cfp_mode_selected,
                                            len + len2 + 1);
-    strcpy(pyenhance->cfp_mode_selected + len, cand_word->strWord);
+    memcpy(pyenhance->cfp_mode_selected + len, cand_word->strWord, len2 + 1);
     FcitxInstanceCommitString(pyenhance->owner,
                               FcitxInstanceGetCurrentIC(pyenhance->owner),
                               pyenhance->cfp_mode_selected);
@@ -99,8 +99,8 @@ CharFromPhraseSetClientPreedit(PinyinEnhance *pyenhance, const char *str)
     FcitxInputState *input = FcitxInstanceGetInputState(instance);
     FcitxMessages *client_preedit = FcitxInputStateGetClientPreedit(input);
     FcitxMessagesSetMessageCount(client_preedit, 0);
-    FcitxMessagesAddMessageAtLast(client_preedit, MSG_INPUT, "%s%s",
-                                  pyenhance->cfp_mode_selected, str);
+    FcitxMessagesAddMessageStringsAtLast(client_preedit, MSG_INPUT,
+                                         pyenhance->cfp_mode_selected, str);
 }
 
 static void
@@ -113,10 +113,11 @@ CharFromPhraseModeUpdateUI(PinyinEnhance *pyenhance)
     char **cur_list = pyenhance->cfp_mode_lists[pyenhance->cfp_mode_cur];
     FcitxCandidateWordSetPage(cand_list, 0);
     FcitxMessagesSetMessageCount(preedit, 0);
-    FcitxMessagesAddMessageAtLast(preedit, MSG_INPUT, "%s (%s)",
-                                  pyenhance->cfp_mode_selected,
-                                  cur_list[0]);
+    FcitxMessagesAddMessageStringsAtLast(preedit, MSG_INPUT,
+                                         pyenhance->cfp_mode_selected, " (",
+                                         cur_list[0], ")");
     CharFromPhraseSetClientPreedit(pyenhance, *(++cur_list));
+    FcitxInputStateSetShowCursor(input, false);
     int i;
     FcitxCandidateWord *cand_word;
     /* use existing cand_word added before if they exist */
@@ -289,8 +290,7 @@ CharFromPhraseModePre(PinyinEnhance *pyenhance, FcitxKeySym sym,
     if (index >= 0)
         return FcitxCandidateWordChooseByIndex(cand_list, index);
     if (FcitxHotkeyIsHotKey(sym, state, config->hkPrevPage)) {
-        if (FcitxCandidateWordHasPrev(cand_list)) {
-            FcitxCandidateWordGoPrevPage(cand_list);
+        if (FcitxCandidateWordGoPrevPage(cand_list)) {
             CharFromPhraseSyncPreedit(pyenhance, cand_list);
             return IRV_FLAG_UPDATE_INPUT_WINDOW;
         }
@@ -301,8 +301,7 @@ CharFromPhraseModePre(PinyinEnhance *pyenhance, FcitxKeySym sym,
         }
         return IRV_DO_NOTHING;
     } else if (FcitxHotkeyIsHotKey(sym, state, config->hkNextPage)) {
-        if (FcitxCandidateWordHasNext(cand_list)) {
-            FcitxCandidateWordGoNextPage(cand_list);
+        if (FcitxCandidateWordGoNextPage(cand_list)) {
             CharFromPhraseSyncPreedit(pyenhance, cand_list);
             return IRV_FLAG_UPDATE_INPUT_WINDOW;
         }
@@ -325,9 +324,10 @@ CharFromPhraseModePre(PinyinEnhance *pyenhance, FcitxKeySym sym,
     } else if (FcitxHotkeyIsHotKey(sym, state, FCITX_ENTER)) {
         int len = strlen(pyenhance->cfp_mode_selected);
         char *cur_full = *pyenhance->cfp_mode_lists[pyenhance->cfp_mode_cur];
+        int len2 = strlen(cur_full);
         pyenhance->cfp_mode_selected = realloc(pyenhance->cfp_mode_selected,
-                                               len + strlen(cur_full) + 1);
-        strcpy(pyenhance->cfp_mode_selected + len, cur_full);
+                                               len + len2 + 1);
+        memcpy(pyenhance->cfp_mode_selected + len, cur_full, len2 + 1);
         FcitxInstanceCommitString(pyenhance->owner,
                                   FcitxInstanceGetCurrentIC(pyenhance->owner),
                                   pyenhance->cfp_mode_selected);
@@ -364,10 +364,8 @@ CharFromPhraseModeListFromWord(const char *word)
         return NULL;
     do {
         if ((len = p - word) > 1) {
-            buff[n] = malloc(len + 1);
+            buff[n] = fcitx_utils_set_str_with_len(NULL, word, len);
             strncat(full, word, len);
-            memcpy(buff[n], word, len);
-            buff[n][len] = '\0';
             n++;
         }
         if (!*p)
