@@ -47,26 +47,44 @@ CONFIG_BINDING_REGISTER("SkinFont", "OtherColor", skinFont.fontColor[MSG_OTHER])
 CONFIG_BINDING_REGISTER("SkinFont", "ActiveMenuColor", skinFont.menuFontColor[MENU_ACTIVE])
 CONFIG_BINDING_REGISTER("SkinFont", "InactiveMenuColor", skinFont.menuFontColor[MENU_INACTIVE])
 
-CONFIG_BINDING_REGISTER("SkinMainBar", "BackImg", skinMainBar.backImg)
+CONFIG_BINDING_REGISTER("SkinMainBar", "BackImg", skinMainBar.background.background)
+CONFIG_BINDING_REGISTER("SkinMainBar", "Overlay", skinMainBar.background.overlay)
+CONFIG_BINDING_REGISTER("SkinMainBar", "OverlayDock", skinMainBar.background.dock)
+CONFIG_BINDING_REGISTER("SkinMainBar", "OverlayOffsetX", skinMainBar.background.overlayOffsetX)
+CONFIG_BINDING_REGISTER("SkinMainBar", "OverlayOffsetY", skinMainBar.background.overlayOffsetY)
+CONFIG_BINDING_REGISTER("SkinMainBar", "MarginTop", skinMainBar.background.marginTop)
+CONFIG_BINDING_REGISTER("SkinMainBar", "MarginBottom", skinMainBar.background.marginBottom)
+CONFIG_BINDING_REGISTER("SkinMainBar", "MarginLeft", skinMainBar.background.marginLeft)
+CONFIG_BINDING_REGISTER("SkinMainBar", "MarginRight", skinMainBar.background.marginRight)
+CONFIG_BINDING_REGISTER("SkinMainBar", "ClickMarginTop", skinMainBar.background.clickMarginTop)
+CONFIG_BINDING_REGISTER("SkinMainBar", "ClickMarginBottom", skinMainBar.background.clickMarginBottom)
+CONFIG_BINDING_REGISTER("SkinMainBar", "ClickMarginLeft", skinMainBar.background.clickMarginLeft)
+CONFIG_BINDING_REGISTER("SkinMainBar", "ClickMarginRight", skinMainBar.background.clickMarginRight)
+CONFIG_BINDING_REGISTER("SkinMainBar", "FillVertical", skinMainBar.background.fillV)
+CONFIG_BINDING_REGISTER("SkinMainBar", "FillHorizontal", skinMainBar.background.fillH)
 CONFIG_BINDING_REGISTER("SkinMainBar", "Logo", skinMainBar.logo)
 CONFIG_BINDING_REGISTER("SkinMainBar", "Eng", skinMainBar.eng)
 CONFIG_BINDING_REGISTER("SkinMainBar", "Active", skinMainBar.active)
-CONFIG_BINDING_REGISTER("SkinMainBar", "MarginLeft", skinMainBar.marginLeft)
-CONFIG_BINDING_REGISTER("SkinMainBar", "MarginRight", skinMainBar.marginRight)
-CONFIG_BINDING_REGISTER("SkinMainBar", "MarginTop", skinMainBar.marginTop)
-CONFIG_BINDING_REGISTER("SkinMainBar", "MarginBottom", skinMainBar.marginBottom)
-CONFIG_BINDING_REGISTER("SkinMainBar", "FillVertical", skinMainBar.fillV)
-CONFIG_BINDING_REGISTER("SkinMainBar", "FillHorizontal", skinMainBar.fillH)
 CONFIG_BINDING_REGISTER_WITH_FILTER("SkinMainBar", "Placement", skinMainBar.placement, FilterPlacement)
 CONFIG_BINDING_REGISTER("SkinMainBar", "UseCustomTextIconColor", skinMainBar.bUseCustomTextIconColor)
 CONFIG_BINDING_REGISTER("SkinMainBar", "ActiveTextIconColor", skinMainBar.textIconColor[0])
 CONFIG_BINDING_REGISTER("SkinMainBar", "InactiveTextIconColor", skinMainBar.textIconColor[1])
 
-CONFIG_BINDING_REGISTER("SkinInputBar", "BackImg", skinInputBar.backImg)
-CONFIG_BINDING_REGISTER("SkinInputBar", "MarginTop", skinInputBar.marginTop)
-CONFIG_BINDING_REGISTER("SkinInputBar", "MarginBottom", skinInputBar.marginBottom)
-CONFIG_BINDING_REGISTER("SkinInputBar", "MarginLeft", skinInputBar.marginLeft)
-CONFIG_BINDING_REGISTER("SkinInputBar", "MarginRight", skinInputBar.marginRight)
+CONFIG_BINDING_REGISTER("SkinInputBar", "BackImg", skinInputBar.background.background)
+CONFIG_BINDING_REGISTER("SkinInputBar", "Overlay", skinInputBar.background.overlay)
+CONFIG_BINDING_REGISTER("SkinInputBar", "OverlayDock", skinInputBar.background.dock)
+CONFIG_BINDING_REGISTER("SkinInputBar", "OverlayOffsetX", skinInputBar.background.overlayOffsetX)
+CONFIG_BINDING_REGISTER("SkinInputBar", "OverlayOffsetY", skinInputBar.background.overlayOffsetY)
+CONFIG_BINDING_REGISTER("SkinInputBar", "MarginTop", skinInputBar.background.marginTop)
+CONFIG_BINDING_REGISTER("SkinInputBar", "MarginBottom", skinInputBar.background.marginBottom)
+CONFIG_BINDING_REGISTER("SkinInputBar", "MarginLeft", skinInputBar.background.marginLeft)
+CONFIG_BINDING_REGISTER("SkinInputBar", "MarginRight", skinInputBar.background.marginRight)
+CONFIG_BINDING_REGISTER("SkinInputBar", "ClickMarginTop", skinInputBar.background.clickMarginTop)
+CONFIG_BINDING_REGISTER("SkinInputBar", "ClickMarginBottom", skinInputBar.background.clickMarginBottom)
+CONFIG_BINDING_REGISTER("SkinInputBar", "ClickMarginLeft", skinInputBar.background.clickMarginLeft)
+CONFIG_BINDING_REGISTER("SkinInputBar", "ClickMarginRight", skinInputBar.background.clickMarginRight)
+CONFIG_BINDING_REGISTER("SkinInputBar", "FillVertical", skinInputBar.background.fillV)
+CONFIG_BINDING_REGISTER("SkinInputBar", "FillHorizontal", skinInputBar.background.fillH)
 CONFIG_BINDING_REGISTER("SkinInputBar", "CursorColor", skinInputBar.cursorColor)
 CONFIG_BINDING_REGISTER("SkinInputBar", "InputPos", skinInputBar.iInputPos)
 CONFIG_BINDING_REGISTER("SkinInputBar", "OutputPos", skinInputBar.iOutputPos)
@@ -76,21 +94,27 @@ CONFIG_BINDING_REGISTER("SkinInputBar", "BackArrowX", skinInputBar.iBackArrowX)
 CONFIG_BINDING_REGISTER("SkinInputBar", "BackArrowY", skinInputBar.iBackArrowY)
 CONFIG_BINDING_REGISTER("SkinInputBar", "ForwardArrowX", skinInputBar.iForwardArrowX)
 CONFIG_BINDING_REGISTER("SkinInputBar", "ForwardArrowY", skinInputBar.iForwardArrowY)
-CONFIG_BINDING_REGISTER("SkinInputBar", "FillVertical", skinInputBar.fillV)
-CONFIG_BINDING_REGISTER("SkinInputBar", "FillHorizontal", skinInputBar.fillH)
 
 CONFIG_BINDING_REGISTER("SkinTrayIcon", "Active", skinTrayIcon.active)
 CONFIG_BINDING_REGISTER("SkinTrayIcon", "Inactive", skinTrayIcon.inactive)
 
-CONFIG_BINDING_REGISTER("SkinMenu", "BackImg", skinMenu.backImg)
-CONFIG_BINDING_REGISTER("SkinMenu", "MarginTop", skinMenu.marginTop)
-CONFIG_BINDING_REGISTER("SkinMenu", "MarginBottom", skinMenu.marginBottom)
-CONFIG_BINDING_REGISTER("SkinMenu", "MarginLeft", skinMenu.marginLeft)
-CONFIG_BINDING_REGISTER("SkinMenu", "MarginRight", skinMenu.marginRight)
+CONFIG_BINDING_REGISTER("SkinMenu", "BackImg", skinMenu.background.background)
+CONFIG_BINDING_REGISTER("SkinMenu", "Overlay", skinMenu.background.overlay)
+CONFIG_BINDING_REGISTER("SkinMenu", "OverlayDock", skinMenu.background.dock)
+CONFIG_BINDING_REGISTER("SkinMenu", "OverlayOffsetX", skinMenu.background.overlayOffsetX)
+CONFIG_BINDING_REGISTER("SkinMenu", "OverlayOffsetY", skinMenu.background.overlayOffsetY)
+CONFIG_BINDING_REGISTER("SkinMenu", "MarginTop", skinMenu.background.marginTop)
+CONFIG_BINDING_REGISTER("SkinMenu", "MarginBottom", skinMenu.background.marginBottom)
+CONFIG_BINDING_REGISTER("SkinMenu", "MarginLeft", skinMenu.background.marginLeft)
+CONFIG_BINDING_REGISTER("SkinMenu", "MarginRight", skinMenu.background.marginRight)
+CONFIG_BINDING_REGISTER("SkinMenu", "ClickMarginTop", skinMenu.background.clickMarginTop)
+CONFIG_BINDING_REGISTER("SkinMenu", "ClickMarginBottom", skinMenu.background.clickMarginBottom)
+CONFIG_BINDING_REGISTER("SkinMenu", "ClickMarginLeft", skinMenu.background.clickMarginLeft)
+CONFIG_BINDING_REGISTER("SkinMenu", "ClickMarginRight", skinMenu.background.clickMarginRight)
+CONFIG_BINDING_REGISTER("SkinMenu", "FillVertical", skinMenu.background.fillV)
+CONFIG_BINDING_REGISTER("SkinMenu", "FillHorizontal", skinMenu.background.fillH)
 CONFIG_BINDING_REGISTER("SkinMenu", "ActiveColor", skinMenu.activeColor)
 CONFIG_BINDING_REGISTER("SkinMenu", "LineColor", skinMenu.lineColor)
-CONFIG_BINDING_REGISTER("SkinMenu", "FillVertical", skinMenu.fillV)
-CONFIG_BINDING_REGISTER("SkinMenu", "FillHorizontal", skinMenu.fillH)
 
 CONFIG_BINDING_REGISTER("SkinKeyboard", "BackImg", skinKeyboard.backImg)
 CONFIG_BINDING_REGISTER("SkinKeyboard", "KeyColor", skinKeyboard.keyColor)
